﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;

namespace ParcelFabricSDK
{
  internal class BuildParcels : Button
  {
    protected async override void OnClick()
    {
      var myParcelFabricLayer =
        MapView.Active.Map.GetLayersAsFlattenedList().OfType<ParcelLayer>().FirstOrDefault();
      if (myParcelFabricLayer == null)
      {
        MessageBox.Show("Please add a parcel fabric to the map.", "Copy Line Features To Parcel Type");
        return;
      }
      string sReportResult = "";
      bool bFirstExecuteOK = true;
      string errorMessage = await QueuedTask.Run(async () =>
      {
        var editOper = new EditOperation()
        {
          Name = "Build Parcels",
          ProgressMessage = "Build Parcels...",
          ShowModalMessageAfterFailure = true,
          SelectNewFeatures = true,
          SelectModifiedFeatures = false
        };
        bool bHasActiveRecord = false;
        Guid guid = new Guid();
        try
        {
          var theActiveRecord = myParcelFabricLayer.GetActiveRecord();
          bHasActiveRecord = (theActiveRecord != null);
          if (bHasActiveRecord)
            guid = theActiveRecord.Guid;
          List<FeatureLayer> parcelLayers = new List<FeatureLayer>();
          List<string> sParcelTypes = new List<string> {"Tax", "Lot"};
          foreach (string sParcTyp in sParcelTypes)
          {
            IEnumerable<FeatureLayer> lyrs = 
              await myParcelFabricLayer.GetParcelPolygonLayerByTypeNameAsync(sParcTyp);
            parcelLayers.Add(lyrs.FirstOrDefault());
            lyrs = await myParcelFabricLayer.GetParcelLineLayerByTypeNameAsync(sParcTyp);
            parcelLayers.Add(lyrs.FirstOrDefault());
          }
          IEnumerable<FeatureLayer> ptLyrs = await myParcelFabricLayer.GetPointsLayerAsync();
          parcelLayers.Add(ptLyrs.FirstOrDefault());
          ParcelEditToken peToken = null;
          if (bHasActiveRecord)
            peToken = editOper.BuildParcelsByRecord(myParcelFabricLayer, guid, parcelLayers);
          else
            peToken = editOper.BuildParcelsByExtent(myParcelFabricLayer, MapView.Active.Extent, parcelLayers);
          if (!editOper.Execute())
          {
            bFirstExecuteOK = false;
            return editOper.ErrorMessage;
          }
          var FeatSetCreated = peToken.CreatedFeatures;
          var FeatSetModified = peToken.ModifiedFeatures;

          var editOperation2 = editOper.CreateChainedOperation();
          sReportResult = "";
          Dictionary<string, object> ParcelAttributes = new Dictionary<string, object>();
          foreach (var kvp in FeatSetModified.ToDictionary())
          {
            foreach (long oid in kvp.Value)
            {
              ParcelAttributes.Add("Name", "My" + kvp.Key.Name + " " + oid.ToString());
              editOperation2.Modify(kvp.Key, oid, ParcelAttributes);
              ParcelAttributes.Clear();
            }
            sReportResult += "There were " + kvp.Value.Count().ToString() + " " + kvp.Key.Name + 
            " features modified." + Environment.NewLine;
          }
          foreach (var kvp in FeatSetCreated.ToDictionary())
          {
            foreach (long oid in kvp.Value)
            {
              //ParcelAttributes.Add("Name", "My" + kvp.Key.Name + " " + oid.ToString());
              //editOperation2.Modify(kvp.Key, oid, ParcelAttributes);
              //ParcelAttributes.Clear();
            }
            sReportResult += "There were " + kvp.Value.Count().ToString() + " new " + kvp.Key.Name + 
              " features created." + Environment.NewLine;
          }
          if (!editOperation2.Execute())
            return editOperation2.ErrorMessage;
        }
        catch (Exception ex)
        {
          if (bFirstExecuteOK)//means partial success, so roll-back first edit
          {
            await editOper.UndoAsync(); //TODO: the operation is undone, but still on the undo stack, so remove it
          }
          errorMessage = ex.Message;
        }
        return "";
      });
      if (!string.IsNullOrEmpty(errorMessage))
        MessageBox.Show(errorMessage, "Build Parcels");
      else if (!string.IsNullOrEmpty(sReportResult))
        MessageBox.Show(sReportResult, "Build Parcels");
    }
  }
}
