﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;

namespace ParcelFabricSDK
{
  internal class ChangeParcelType : Button
  {
    protected async override void OnClick()
    {
      var myParcelFabricLayer = 
        MapView.Active.Map.GetLayersAsFlattenedList().OfType<ParcelLayer>().FirstOrDefault();
      if (myParcelFabricLayer == null)
      {
        MessageBox.Show("Please add a parcel fabric to the map.", "Change Parcel Type");
        return;
      }
      string sSourceParcelType = 
        Microsoft.VisualBasic.Interaction.InputBox("Source Parcel Type:", "Change Parcel Type", "Tax");
      if (sSourceParcelType.Trim().Length == 0)
        return;
      string sTargetParcelType = 
        Microsoft.VisualBasic.Interaction.InputBox("Target Parcel Type:", "Change Parcel Type", "Lot");
       if (sTargetParcelType.Trim().Length == 0)
        return;
      string sReportResult = "";
      string errorMessage = await QueuedTask.Run( async () =>
      {
        try
        {
          //get the feature polygon layer for the "Lot" type
          var sourcePolygonLotTypeEnum = 
            await myParcelFabricLayer.GetParcelPolygonLayerByTypeNameAsync(sSourceParcelType);
          var sourcePolygonLotTypeLayer = sourcePolygonLotTypeEnum.FirstOrDefault();
          var targetParcelTypeFeatLyrEnum = 
            await myParcelFabricLayer.GetParcelPolygonLayerByTypeNameAsync(sTargetParcelType);
          var targetParcelTypeFeatLyr = targetParcelTypeFeatLyrEnum.FirstOrDefault();
          if (myParcelFabricLayer == null || sourcePolygonLotTypeLayer == null)
            return "";

          targetParcelTypeFeatLyrEnum =
            await myParcelFabricLayer.GetParcelLineLayerByTypeNameAsync(sTargetParcelType);
          var targetParcelLineTypeFeatLyr = targetParcelTypeFeatLyrEnum.FirstOrDefault();


          var ids = new List<long>(sourcePolygonLotTypeLayer.GetSelection().GetObjectIDs());
          //var kvp00 = new KeyValuePair<MapMember, List<long>>(sourcePolygonLotTypeLayer, ids);
          //var sourceFeatures = new List<KeyValuePair<MapMember, List<long>>> { kvp00 };
          var sourceParcFeats = new Dictionary<MapMember, List<long>>();
          sourceParcFeats.Add(sourcePolygonLotTypeLayer, ids);

          var opCpToPT = new EditOperation()
          {
            Name = "Change Parcel Type",
            ProgressMessage = "Change Parcel Type...",
            ShowModalMessageAfterFailure = true,
            SelectNewFeatures = true,
            SelectModifiedFeatures = false
          };
          var peToken = opCpToPT.ChangeParcelType(myParcelFabricLayer, SelectionSet.FromDictionary(sourceParcFeats),
            targetParcelTypeFeatLyr, targetParcelLineTypeFeatLyr);
          if (!opCpToPT.Execute())
            return opCpToPT.ErrorMessage;

          var FeatSetCreated = peToken.CreatedFeatures;
          var FeatSetModified = peToken.ModifiedFeatures;
          if (FeatSetCreated != null)
          {
            foreach (var kvp in FeatSetCreated.ToDictionary())
            {
              foreach (long oid in kvp.Value)
              {
                //ParcelAttributes.Add("Name", "My" + kvp.Key.Name + " " + oid.ToString());
                //editOperation2.Modify(kvp.Key, oid, ParcelAttributes);
                //ParcelAttributes.Clear();
              }
              sReportResult += "There were " + kvp.Value.Count().ToString() + " new " + kvp.Key.Name + 
              " features created." + Environment.NewLine;
            }
          }
          if (FeatSetModified != null)
          {
            foreach (var kvp in FeatSetModified.ToDictionary())
            {
              foreach (long oid in kvp.Value)
              {
                //ParcelAttributes.Add("Name", "My" + kvp.Key.Name + " " + oid.ToString());
                //editOperation2.Modify(kvp.Key, oid, ParcelAttributes);
                //ParcelAttributes.Clear();
              }
              sReportResult += "There were " + kvp.Value.Count().ToString() + " " + kvp.Key.Name + 
                " features modified." + Environment.NewLine;
            }
          }
        }
        catch (Exception ex)
        {
          return ex.Message;
        }
        return "";
      });
      if (!string.IsNullOrEmpty(errorMessage))
        MessageBox.Show(errorMessage, "Change Parcel Type");
      else if (!string.IsNullOrEmpty(sReportResult))
        MessageBox.Show(sReportResult, "Change Parcel Type");
    }
  }
}
